<?php
/**
 * HTTP Response Header Class
 *
 * HTTP Response Header ClassはHTTP Responseをコントロールするクラスです。
 *
 * @package http_response_header
 * @access public
 * @author http://purl.org/meta/me/
 * @version http_response_header.php 1.0 2011-08-20
 * @license Creative Commons License CC BY (http://creativecommons.org/licenses/by/3.0/)
 */
class HttpResponseHeader {

	/**
	 * HTTP Status Codeの一覧。(ja)
	 *
	 * List of HTTP Status Code.(en)
	 *
	 * @var array
	 * @access public
	 */
	static $httpStatusCodes = array(
		100 => 'Continue',
		101 => 'Switching Protocols',
		200 => 'OK',
		201 => 'Created',
		202 => 'Accepted',
		203 => 'Non-Authoritative Information',
		204 => 'No Content',
		205 => 'Reset Content',
		206 => 'Partial Content',
		226 => 'IM Used',
		300 => 'Multiple Choices',
		301 => 'Moved Permanently',
		302 => 'Found',
		303 => 'See Other',
		304 => 'Not Modified',
		305 => 'Use Proxy',
		307 => 'Temporary Redirect',
		400 => 'Bad Request',
		401 => 'Unauthorized',
		402 => 'Payment Required',
		403 => 'Forbidden',
		404 => 'Not Found',
		405 => 'Method Not Allowed',
		406 => 'Not Acceptable',
		407 => 'Proxy Authentication Required',
		408 => 'Request Time-out',
		409 => 'Conflict',
		410 => 'Gone',
		411 => 'Length Required',
		412 => 'Precondition Failed',
		413 => 'Request Entity Too Large',
		414 => 'Request-URI Too Large',
		415 => 'Unsupported Media Type',
		416 => 'Requested range not satisfiable',
		417 => 'Expectation Failed',
		500 => 'Internal Server Error',
		501 => 'Not Implemented',
		502 => 'Bad Gateway',
		503 => 'Service Unavailable',
		504 => 'Gateway Time-out',
		505 => 'HTTP Version Not Supported',
		509 => 'Bandwidth Limit Exceeded'
	);

	/**
	 * HTTPのバージョン。(ja)
	 *
	 * HTTP Version.(en)
	 *
	 * @var string
	 * @access public
	 */
	static $httpVersion = '1.0';

	/**
	 * HTTPのバージョン。(ja)
	 *
	 * Available HTTP Versions.(en)
	 *
	 * @var array
	 * @access public
	 */
	static $httpVersions = array('1.0', '1.1');

	/**
	 * 有効期限を設定します。(ja)
	 *
	 * Set the expiration date.(en)
	 *
	 * <code>
	 * HttpResponseHeader::expires(60 * 60);
	 *
	 * # 以下のようにレスポンスヘッダーを出力します。
	 * # Cache-Control: max-age=3600
	 * # Expires: Mon, 29 Aug 2011 10:31:06 GMT
	 * </code>
	 *
	 * @param integer $time 有効期限までの秒数(ja) Expiration Seconds(en)
	 * @return void
	 * @access public
	 */
	function expires($time) {
		self::header('Cache-Control', 'max-age=' . $time);

		self::header('Expires', $time + time());
	}

	/**
	 * タイムスタンプをRFC 1123の日付形式に変換します。(ja)
	 *
	 * <code>
	 * HttpResponseHeader::date(1314600068);
	 *
	 * # 出力すると以下のようになります。
	 * # Mon, 29 Aug 2011 06:41:08 GMT
	 * </code>
	 *
	 * @param integer $timestamp タイムスタンプ(ja) Timestamp(en)
	 * @return string RFC 1123の日付形式(ja) RFC 1123 date format(en)
	 * @access public
	 */
	function date($timestamp = null) {
		if(is_null($timestamp)) {
			$timestamp = time();
		}

		return gmdate('D, d M Y H:i:s', $timestamp) . ' GMT';
	}

	/**
	 * 既に出力したHTTP Response Headerを取得します。(ja)
	 *
	 * Get the HTTP Response Header output now.(en)
	 *
	 * @param mixed $name 取得したいフィールドを指定します。全てのフィールドを取得したい場合値は空でよいです。(ja) Specify the field to get.(en)
	 * @return mixed 文字列または配列(ja) string or array(en)
	 * @access public
	 */
	function getHeader($name = null) {
		$array = array();

		$headersList = headers_list();

		foreach($headersList as $key => $value) {
			preg_match('/^(.+?): (.+?)$/', $value, $matches);

			$array[][$matches[1]] = $matches[2];
		}

		$headers = call_user_func_array('array_merge_recursive', $array);

		if($name) {
			if(isset($headers[$name])) {
				return $headers[$name];
			}

			return null;
		}

		return $headers;
	}

	/**
	 * リクエストされたHTTPのバージョン。(ja)
	 *
	 * Version of the HTTP request.(en)
	 *
	 * @return string HTTPのバージョン(ja) HTTP Version(en)
	 * @access public
	 */
	function getHttpVersion() {
		if (isset($_SERVER['SERVER_PROTOCOL'])) {
			return substr($_SERVER['SERVER_PROTOCOL'], -3);
		}

		return self::$httpVersion;
	}

	/**
	 * gzipのエンコードを開始します。(ja)
	 *
	 * Start the encoding of gzip.(en)
	 *
	 * <code>
	 * HttpResponseHeader::gzip();
	 *
	 * # 以下のようにレスポンスヘッダーを出力します。
	 * # Vary: Accept-Encoding
	 * # Content-Encoding: gzip
	 * </code>
	 *
	 * @return mixed 成功した場合はContent-Encoding Headerの内容を、失敗した場合はfalseを返します。
	 * @access public
	 */
	function gzip() {
		ob_start('ob_gzhandler');

		self::header('Vary', 'Accept-Encoding', false);

		if(isset($_SERVER['HTTP_ACCEPT_ENCODING'])) {
			$encodings = explode(',', $_SERVER['HTTP_ACCEPT_ENCODING']);

			foreach($encodings as $value) {
				if(in_array(strtolower(trim($value)), array('gzip', 'deflate'))) {
					self::header('Content-Encoding', $value);

					return $value;
				}
			}
		}

		return false;
	}

	/**
	 * HTTP Response Headerを出力します。(ja)
	 *
	 * Print the HTTP Response Header(en)
	 *
	 * <code>
	 * HttpResponseHeader::header(array(
	 *     'ETag' => 'Etag Field value',
	 *     'Content-MD5' => 'Content-MD5 Field value',
	 *     'Last-Modified' => 1314600068
	 * ));
	 *
	 * # 以下のようにレスポンスヘッダーを出力します。
	 * # Etag: "d4289a42770d87be5a78cbcefcc1e3b76a07f94dd1005020736b6943e0e65bf07d97472739fb775ea7a8107ce024864def444d935f494e096f2bfd0382bb2e84"
	 * # Content-MD5: RhQI+teVkAukVCltE3K+yg==
	 * # Last-Modified: Mon, 29 Aug 2011 06:41:08 GMT
	 * </code>
	 *
	 * @param mixed $headers Headerの配列またはフィールド名(ja) Headers array or field name(en)
	 * @param mixed $replace 前に送信された類似のヘッダを置換するか。$headersがフィールド名の場合フィールドの内容
 	 * @param boolean $_replace
	 * @return void
	 * @access public
	 */
	function header($headers, $replace = true, $_replace = true) {
		if(!is_array($headers)) {
			$headers = array($headers => $replace);

			$replace = $_replace;
		}

		foreach($headers as $key => $value) {
			if(is_array($value)) {
				$firstKey = reset(array_keys($value));

				foreach($value as $_key => $_value) {
					if($firstKey === $_key) {
						self::header($key, $_value, $replace);
					} else {
						self::header($key, $_value, false);
					}
				}
			} else {
				if(($key === 'Date' || $key === 'Expires' || $key === 'Last-Modified') && (is_numeric($value) || is_null($value))) {
					$value = self::date($value);
				} elseif($key === 'ETag' && !preg_match('/^(W\/)?"[a-zA-Z0-9\-]+"$/', $value)) {
					$value = '"' . hash('sha512', $value) . '"';
				} elseif($key === 'Content-MD5' && strlen($value) !== 24) {
					$value = base64_encode(md5($value, true));
				}

				header($key . ': ' . $value, $replace);
			}
		}
	}

	/**
	 * HTML用のHeaderを出力します。(ja)
	 *
	 * Print the Header for HTML.(en)
	 *
	 * <code>
	 * HttpResponseHeader::htmlHeader();
	 *
	 * # 以下のようにレスポンスヘッダーを出力します。
	 * # Content-Style-Type: text/css
	 * # Content-Script-Type: text/javascript
	 * </code>
	 *
	 * @return void
	 * @access public
	 */
	function htmlHeader() {
		self::header(array(
			'Content-Style-Type' => 'text/css',
			'Content-Script-Type' => 'text/javascript',
		));
	}

	/**
	 * Webブラウザーがキャッシュしない様に指示します。(ja)
	 *
	 * Instructs the Web browser not to cache.(en)
	 *
	 * <code>
	 * HttpResponseHeader::noCache();
	 *
	 * # 以下のようにレスポンスヘッダーを出力します。
	 * # Expires: Mon, 26 Jul 1997 05:00:00 GMT
	 * # Last-Modified: Mon, 29 Aug 2011 07:15:22 GMT
	 * # Cache-Control: no-store, no-cache, must-revalidate, post-check=0, pre-check=0
	 * # Pragma: no-cache
	 * </code>
	 *
	 * @return void
	 * @access public
	 */
	function noCache() {
		self::header(array(
			'Expires' => 'Mon, 26 Jul 1997 05:00:00 GMT',
			'Last-Modified' => null,
			'Cache-Control' => array(
				'no-store, no-cache, must-revalidate',
				'post-check=0, pre-check=0'
			),
			'Pragma' => 'no-cache'
		));
	}

	/**
	 * 条件が一致すれば"304 Not Modified"または"412 Precondition Failed"を出力します。(ja)
	 *
	 * Print the "304 Not Modified" or "412 Precondition Failed" if the conditions are matched.(en)
	 *
	 * @param boolean $exit PHPを終了するならばtrue(ja) If True, the closing PHP(en)
	 * @return mixed
	 * @access public
	 */
	function notModified($exit = true) {
		$headers = self::getHeader();

		$notModified = false;

		if(isset($headers['ETag']) && isset($_SERVER['HTTP_IF_MATCH']) && $headers['ETag'] !== $_SERVER['HTTP_IF_MATCH']) {
			self::statusCode(412);

			return 412;
		} elseif(isset($headers['Last-Modified']) && isset($_SERVER['HTTP_IF_UNMODIFIED_SINCE']) && strtotime($_SERVER['HTTP_IF_UNMODIFIED_SINCE']) !== strtotime($headers['Last-Modified'])) {
			self::statusCode(412);

			return 412;
		}

		if(isset($headers['ETag']) && isset($_SERVER['HTTP_IF_NONE_MATCH']) && $headers['ETag'] === $_SERVER['HTTP_IF_NONE_MATCH']) {
			if(isset($_SERVER['REQUEST_METHOD']) && $_SERVER['REQUEST_METHOD'] === 'GET' || $_SERVER['REQUEST_METHOD'] === 'HEAD') {
				$notModified = true;
			} else {
				self::statusCode(412);

				return 412;
			}
		} elseif(isset($headers['Last-Modified']) && isset($_SERVER['HTTP_IF_MODIFIED_SINCE']) && strtotime($_SERVER['HTTP_IF_MODIFIED_SINCE']) === strtotime($headers['Last-Modified'])) {
			$notModified = true;
		}

		if($notModified) {
			self::statusCode(304);

			if($exit) {
				ob_end_clean();
				exit;
			}

			return 304;
		}

		return false;
	}

	/**
	 * パブリックなキャッシュをするように指示します。(ja)
	 *
	 * Public Cache(en)
	 *
	 * <code>
	 * HttpResponseHeader::publicCache();
	 *
	 * # 以下のようにレスポンスヘッダーを出力します。
	 * # Cache-Control: public
	 * # Pragma: public
	 * </code>
	 *
	 * @return void
	 * @access public
	 */
	function publicCache() {
		self::header(array(
			'Cache-Control' => 'public',
			'Pragma' => 'public'
		));
	}

	/**
	 * 範囲リクエストに対応します。(ja)
	 *
	 * Corresponds to a Range Request.(en)
	 *
	 * @param string $content
	 * @return mixed
	 * @access public
	 */
	function range($content = null) {
		HttpResponseHeader::header('Accept-Ranges', 'bytes');

		if(isset($_SERVER['HTTP_RANGE']) && preg_match('/^bytes=([0-9]+)?-([0-9]+)?$/', $_SERVER['HTTP_RANGE'], $matches)) {
			$headers = self::getHeader();

			if(isset($_SERVER['HTTP_IF_RANGE'])) {
				if(preg_match('/^(W\/)?"[a-zA-Z0-9\-]+"$/', $_SERVER['HTTP_IF_RANGE']) && isset($headers['ETag']) && $headers['ETag'] !== $_SERVER['HTTP_IF_RANGE']) {
					return false;
				} elseif(isset($headers['Last-Modified']) && strtotime($_SERVER['HTTP_IF_RANGE']) !== strtotime($headers['Last-Modified'])) {
					return false;
				} elseif(!isset($headers['ETag']) && !isset($headers['Last-Modified'])) {
					return false;
				}
			}

			$od = false;

			if(!$content) {
				$content = ob_get_contents();

				$od = true;
			}

			$size = strlen($content);

			if(isset($matches[1]) && $matches[1] === '') {
				$matches[1] = null;
			}

			if(isset($matches[2]) && $matches[2] === '') {
				$matches[2] = null;
			}

			if(isset($matches[1]) && isset($matches[2])) {
				$offset = $matches[1];

				$length = $matches[2] - $offset + 1;

				$total = $matches[2];

				if($total >= $size) {
					$total = $size - 1;
				}
			} else {
				$length = $size;

				$total = $size - 1;

				if(isset($matches[1]) && !isset($matches[2])) {
					$offset = $matches[1];
				} elseif(!isset($matches[1]) && isset($matches[2])) {
					$offset = $size - $matches[2];
				} else {
					self::statusCode(416);

					return 416;
				}
			}

			if($offset >= $size || $offset > $total) {
				self::statusCode(416);

				return 416;
			}

			self::statusCode(206);

			self::header('Content-Range', 'bytes ' . $offset . '-' . $total . '/' . $size);

			$content = substr($content, $offset, $length);

			if($od) {
				ob_clean();
			}

			echo $content;

			return 206;
		}

		return false;
	}

	/**
	 * Serverの情報を出力します。(ja)
	 *
	 * Print the Server information.(en)
	 *
	 * <code>
	 * HttpResponseHeader::server('Apache/2.2');
	 *
	 * # 以下のようにレスポンスヘッダーを出力します。
	 * # Server: Apache/2.2
	 * </code>
	 *
	 * @param string $content
	 * @return boolean 成功ならばtrueを、失敗ならばfalseを返す。
	 * @access public
	 */
	function server($content = null) {
		if($content) {
			self::header('Server', $content);

			return true;
		} elseif(isset($_SERVER['SERVER_SOFTWARE'])) {
			self::header('Server', $_SERVER['SERVER_SOFTWARE']);

			return true;
		}

		return false;
	}

	/**
	 * HTTP Status Codeを出力します。(ja)
	 *
	 * Print the HTTP Status Code.(en)
	 *
	 * <code>
	 * HttpResponseHeader::statusCode(404);
	 *
	 * # 以下のようにレスポンスヘッダーを出力します。
	 * # HTTP/1.1 404 Not Found
	 * </code>
	 *
	 * @param integer $status HTTP Status Codeの番号(ja) The number of HTTP Status Code(en)
	 * @param string $httpVersion HTTPのバージョン(ja) HTTP Version(en)
	 * @return boolean 成功ならばtrueを、失敗ならばfalseを返します。
	 * @access public
	 */
	function statusCode($status = 200, $httpVersion = null) {
		if(!$httpVersion) {
			$httpVersion = self::getHttpVersion();
		}

		if(isset(self::$httpStatusCodes[$status])) {
			header('HTTP/' . $httpVersion . ' ' . $status . ' ' . self::$httpStatusCodes[$status]);

			return true;
		}

		return false;
	}
}
?>